@@ -5,8 +5,9 @@ use warnings;
 
 use URI ();
 use URI::Escape qw(uri_unescape);
+use Scalar::Util ();
 
-our $VERSION = '5.27';
+our $VERSION = '5.28';
 
 sub query
 {
@@ -34,7 +35,7 @@ sub query_form {
         # Try to set query string
         my $delim;
         my $r = $_[0];
-        if (ref($r) eq "ARRAY") {
+        if (_is_array($r)) {
             $delim = $_[1];
             @_ = @$r;
         }
@@ -49,7 +50,7 @@ sub query_form {
             $key = '' unless defined $key;
 	    $key =~ s/([;\/?:@&=+,\$\[\]%])/ URI::Escape::escape_char($1)/eg;
 	    $key =~ s/ /+/g;
-	    $vals = [ref($vals) eq "ARRAY" ? @$vals : $vals];
+	    $vals = [_is_array($vals) ? @$vals : $vals];
             for my $val (@$vals) {
                 if (defined $val) {
                     $val =~ s/([;\/?:@&=+,\$\[\]%])/ URI::Escape::escape_char($1)/eg;
@@ -86,7 +87,7 @@ sub query_keywords
     if (@_) {
         # Try to set query string
 	my @copy = @_;
-	@copy = @{$copy[0]} if @copy == 1 && ref($copy[0]) eq "ARRAY";
+	@copy = @{$copy[0]} if @copy == 1 && _is_array($copy[0]);
 	for (@copy) { s/([;\/?:@&=+,\$\[\]%])/ URI::Escape::escape_char($1)/eg; }
 	$self->query(@copy ? join('+', @copy) : undef);
     }
@@ -114,7 +115,7 @@ sub query_param {
     if (@_) {
         my @new = @old;
         my @new_i = @i;
-        my @vals = map { ref($_) eq 'ARRAY' ? @$_ : $_ } @_;
+        my @vals = map { _is_array($_) ? @$_ : $_ } @_;
 
         while (@new_i > @vals) {
             splice @new, pop @new_i, 2;
@@ -139,7 +140,7 @@ sub query_param {
 sub query_param_append {
     my $self = shift;
     my $key = shift;
-    my @vals = map { ref $_ eq 'ARRAY' ? @$_ : $_ } @_;
+    my @vals = map { _is_array($_) ? @$_ : $_ } @_;
     $self->query_form($self->query_form, $key => \@vals);  # XXX
     return;
 }
@@ -168,7 +169,7 @@ sub query_form_hash {
     while (my($k, $v) = splice(@old, 0, 2)) {
         if (exists $hash{$k}) {
             for ($hash{$k}) {
-                $_ = [$_] unless ref($_) eq "ARRAY";
+                $_ = [$_] unless _is_array($_);
                 push(@$_, $v);
             }
         }
@@ -179,4 +180,15 @@ sub query_form_hash {
     return \%hash;
 }
 
+sub _is_array {
+    return(
+        defined($_[0]) &&
+        ( Scalar::Util::reftype($_[0]) || '' ) eq "ARRAY" && 
+        !(
+            Scalar::Util::blessed( $_[0] ) && 
+            overload::Method( $_[0], '""' )
+        )
+    );
+}
+
 1;
